@@ -191,6 +191,35 @@ AContainer::AContainer( AWindow *twin, int txelems, int tyelems )
   ergWidths = ergHeights = NULL;
 }
 
+AContainer::AContainer( AWindow *twin,
+                        int txelems,
+                        int tyelems,
+                        int border_width,
+                        int min_space,
+                        int max_space )
+{
+    win = twin;
+    xelems = txelems;
+    if ( xelems < 1 ) xelems = 1;
+    yelems = tyelems;
+    if ( yelems < 1 ) yelems = 1;
+    
+    elements = new AContElem[ xelems * yelems ];
+    parent = NULL;
+    x = y = width = height = 0;
+
+    BorderWidth = border_width;
+    if ( BorderWidth < 0 ) BorderWidth = 0;
+
+    MinSpace = min_space;
+    if ( MinSpace < 0 ) MinSpace = 0;
+
+    MaxSpace = max_space;
+    if ( MaxSpace < -1 ) MaxSpace = -1;
+
+    ergWidths = ergHeights = NULL;
+}
+
 AContainer::~AContainer()
 {
   int i;
