@@ -1,6 +1,6 @@
 /* aguix.cc
  * This file belongs to Worker, a filemanager for UNIX/X11.
- * Copyright (C) 2001-2007 Ralf Hoffmann.
+ * Copyright (C) 2001-2008 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "aguix.h"
@@ -35,17 +35,17 @@
 
 #ifndef HAVE_XSETIMVALUES_PROTOTYPE
 extern "C" {
-  extern char *XSetIMValues( XIM, ... );
+  char *XSetIMValues( XIM, ... );
 }
 #endif
 
 #include <X11/Xlocale.h>
 
-/* ChosseButterStyle is based on an example program I found somewhere
+/* ChosseBetterStyle is based on an example program I found somewhere
    But there's was no copyright and the orig codepiece also
    doesn't work...
 */
-XIMStyle ChooseBetterStyle( XIMStyle style1, XIMStyle style2 )
+static XIMStyle ChooseBetterStyle( XIMStyle style1, XIMStyle style2 )
 {
   XIMStyle s, t;
   XIMStyle preedit = XIMPreeditArea |
@@ -88,14 +88,14 @@ XIMStyle ChooseBetterStyle( XIMStyle style1, XIMStyle style2 )
 }
 
 #ifdef XIM_XREGISTER_OKAY
-void AGUIX_im_inst_callback( Display *calldsp, XPointer client_data, XPointer call_data )
+static void AGUIX_im_inst_callback( Display *calldsp, XPointer client_data, XPointer call_data )
 {
   if ( client_data != NULL ) {
     ((AGUIX*)client_data)->IMInstCallback( calldsp );
   }
 }
 
-void AGUIX_im_dest_callback( XIM callim, XPointer client_data, XPointer call_data )
+static void AGUIX_im_dest_callback( XIM callim, XPointer client_data, XPointer call_data )
 {
   if ( client_data != NULL ) {
     ((AGUIX*)client_data)->IMDestCallback();
@@ -109,7 +109,7 @@ long AGUIX::timerEvent = 0;
 std::map<std::string,int> AGUIX::_colors_for_widgets;
 
 #ifdef USE_AGUIXTIMER
-void aguix_timer( int s )
+static void aguix_timer( int s )
 {
   if ( s == SIGALRM ) {
     AGUIX::timerEvent++;
@@ -128,7 +128,8 @@ AGUIX::AGUIX( int argc, char **argv, const std::string &classname ) : m_argc( ar
   initOK=False;
   mainfont=NULL;
   dsp=NULL;
-  colors=0;
+  m_user_colors = 0;
+  m_system_colors = 0;
   privatecmap=false;
   cutstart=NULL;
   cutbuffer=NULL;
@@ -287,10 +288,8 @@ void AGUIX::closeX()
       delete tf;
   }
 
-  while ( colors > 0 ) {
-    removeLastColor();
-  }
-  
+  freeColors();
+    
   if(privatecmap==true) {
     XFreeColormap(dsp,cmap);
   }
@@ -338,50 +337,30 @@ int AGUIX::getCharHeight() const
   return CharHeight;
 }
 
-void AGUIX::setFG(int color)
+void AGUIX::setFG( const AGUIXColor &color )
 {
-  if(color<0) color=0;
-  if(color<colors) {
-    XSetForeground(dsp,gc, getPixel( color ) );
-  } else {
-    XSetForeground(dsp,gc, getPixel( colors - 1 ) );
-  }
+    XSetForeground( dsp, gc, getPixel( color ) );
 }
 
-void AGUIX::setFG(GC tgc,int color)
+void AGUIX::setFG( GC tgc, const AGUIXColor &color )
 {
-  if(tgc==0) setFG(color);
-  else {
-    if(color<0) color=0;
-    if(color<colors) {
-      XSetForeground(dsp,tgc, getPixel( color ) );
-    } else {
-      XSetForeground(dsp,tgc, getPixel( colors - 1 ) );
+    if ( tgc == 0 ) setFG( color );
+    else {
+        XSetForeground( dsp, tgc, getPixel( color ) );
     }
-  }
 }
 
-void AGUIX::setBG(int color)
+void AGUIX::setBG( const AGUIXColor &color )
 {
-  if(color<0) color=0;
-  if(color<colors) {
-    XSetBackground(dsp,gc, getPixel( color ) );
-  } else {
-    XSetBackground(dsp,gc, getPixel( colors - 1 ) );
-  }
+    XSetBackground( dsp, gc, getPixel( color ) );
 }
 
-void AGUIX::setBG(GC tgc,int color)
+void AGUIX::setBG( GC tgc, const AGUIXColor &color )
 {
-  if(tgc==0) setBG(color);
-  else {
-    if(color<0) color=0;
-    if(color<colors) {
-      XSetBackground(dsp,tgc, getPixel( color ) );
-    } else {
-      XSetBackground(dsp,tgc, getPixel( colors - 1 ) );
+    if ( tgc == 0 ) setBG( color );
+    else {
+        XSetBackground( dsp, tgc, getPixel( color ) );
     }
-  }
 }
 
 void AGUIX::FillRectangle(Drawable buffer,int x,int y,int w,int h)
@@ -400,12 +379,12 @@ void AGUIX::FillRectangle(Drawable buffer,GC tgc,int x,int y,int w,int h)
   else XFillRectangle(dsp,buffer,tgc,x,y,w,h);
 }
 
-void AGUIX::DrawText( DrawableCont &dc, const char *text, int x, int y, int color )
+void AGUIX::DrawText( DrawableCont &dc, const char *text, int x, int y, const AGUIXColor &color )
 {
   DrawText( dc, mainfont, text, x, y, color );
 }
 
-void AGUIX::DrawText( DrawableCont &dc, AGUIXFont *tf, const char *text, int x, int y, int color )
+void AGUIX::DrawText( DrawableCont &dc, AGUIXFont *tf, const char *text, int x, int y, const AGUIXColor &color )
 {
   if ( tf == NULL ) {
     mainfont->drawText( dc, text, x, y, color );
@@ -419,27 +398,36 @@ int AGUIX::getFontBaseline() const
   return mainfont->getBaseline();
 }
 
-int AGUIX::AddColor(int red,int green,int blue)
+AGUIXColor AGUIX::AddColor( int red, int green, int blue, AGUIXColor::color_type_t type )
 {
-  if(colors<256) {
+    AGUIXColor ret_col( -1, type );
+
+    if ( type == AGUIXColor::SYSTEM_COLOR && m_system_colors >= 16 ) return ret_col;
+    if ( type == AGUIXColor::USER_COLOR && m_user_colors >= 256 ) return ret_col;
+
 #ifndef HAVE_XFT
     XColor col;
-    col.flags=DoRed|DoGreen|DoBlue;
-    col.red=red*256;
-    col.green=green*256;
-    col.blue=blue*256;
-    if(!XAllocColor(dsp,cmap,&col)) {
-      // error
-      if(privatecmap==true) return -1;
-      else {
-        // now try to go to a private colormap
-	changeColormap();
-	return AddColor(red,green,blue);
-      }
+    col.flags = DoRed | DoGreen | DoBlue;
+    col.red = red * 256;
+    col.green = green * 256;
+    col.blue = blue * 256;
+    
+    if ( !XAllocColor( dsp, cmap, &col ) ) {
+        // error
+        if ( privatecmap == true ) return -1;
+        else {
+            // now try to go to a private colormap
+            changeColormap();
+            return AddColor( red, green, blue, type );
+        }
     }
-    ColBuf[colors++]=col.pixel;
-    if ( colors == 1 ) {
-      prepareExtColors( red, green, blue );
+
+    if ( type == AGUIXColor::SYSTEM_COLOR ) {
+        ret_col = AGUIXColor( m_system_colors++, type );
+        m_system_col_buf[ret_col.getColor()] = col.pixel;
+    } else {
+        ret_col = AGUIXColor( m_user_colors++, type );
+        m_user_col_buf[ret_col.getColor()] = col.pixel;
     }
 #else
     XRenderColor col;
@@ -447,46 +435,71 @@ int AGUIX::AddColor(int red,int green,int blue)
     col.green = green * 256;
     col.blue = blue * 256;
     col.alpha = 0xffff;
+
     if ( ! XftColorAllocValue ( dsp,
                                 DefaultVisual( dsp, scr ),
                                 cmap,
                                 &col,
-                                &ColBuf[colors] ) ) {
-      // error
-      if ( privatecmap == true ) return -1;
-      else {
-        // now try to go to a private colormap
-	changeColormap();
-	return AddColor( red, green, blue );
-      }
+                                ( type == AGUIXColor::SYSTEM_COLOR ) ? &m_system_col_buf[m_system_colors] : &m_user_col_buf[m_user_colors] ) ) {
+        // error
+        if ( privatecmap == true ) return -1;
+        else {
+            // now try to go to a private colormap
+            changeColormap();
+            return AddColor( red, green, blue, type );
+        }
     }
-    colors++;
-    if ( colors == 1 ) {
-      prepareExtColors( red, green, blue );
+
+    if ( type == AGUIXColor::SYSTEM_COLOR ) {
+        ret_col = AGUIXColor( m_system_colors++, type );
+    } else {
+        ret_col = AGUIXColor( m_user_colors++, type );
     }
 #endif
-  }
-  return colors - 1;
+    if ( ret_col.getColor() >= 0 ) {
+        m_color_values[ret_col] = col_values_t( red, green, blue );
+    }
+
+    if ( type == AGUIXColor::USER_COLOR ) {
+        updateSystemColors( m_user_colors - 1 );
+    }
+    return ret_col;
 }
 
-void AGUIX::removeLastColor()
+void AGUIX::freeColors()
 {
-  if(colors>0) {
+    while ( m_user_colors > 0 ) {
 #ifdef HAVE_XFT
-    XftColorFree( dsp,
-                  DefaultVisual( dsp, scr ),
-                  cmap,
-                  &ColBuf[colors - 1] );
+        XftColorFree( dsp,
+                      DefaultVisual( dsp, scr ),
+                      cmap,
+                      &m_user_col_buf[ m_user_colors - 1 ] );
 #else
-    XFreeColors(dsp,cmap,ColBuf+colors-1,1,0);
+        XFreeColors( dsp, cmap, m_user_col_buf + m_user_colors - 1, 1, 0 );
 #endif
-    colors--;
-  }
+        m_user_colors--;
+    }
+
+    while ( m_system_colors > 0 ) {
+#ifdef HAVE_XFT
+        XftColorFree( dsp,
+                      DefaultVisual( dsp, scr ),
+                      cmap,
+                      &m_system_col_buf[ m_system_colors - 1 ] );
+#else
+        XFreeColors( dsp, cmap, m_system_col_buf + m_system_colors - 1, 1, 0 );
+#endif
+        m_system_colors--;
+    }
 }
 
-int AGUIX::getMaxCols() const
+int AGUIX::getNumberOfColorsForType( AGUIXColor::color_type_t type ) const
 {
-  return colors;
+    if ( type == AGUIXColor::SYSTEM_COLOR ) {
+        return m_system_colors;
+    } else {
+        return m_user_colors;
+    }
 }
 
 std::string AGUIX::getClassname() const
@@ -494,30 +507,69 @@ std::string AGUIX::getClassname() const
     return m_classname;
 }
 
-unsigned long AGUIX::getPixel( int color ) const
+unsigned long AGUIX::getPixel( const AGUIXColor &color ) const
 {
-  if ( color < colors && color >= 0 ) {
+    int col_pos;
+    switch( color.getColorType() ) {
+      case AGUIXColor::USER_COLOR:
+          col_pos = color.getColor();
+          if ( col_pos >= m_user_colors ) col_pos = m_user_colors - 1;
+          if ( col_pos < 0 ) col_pos = 0;
+          if ( col_pos >= m_user_colors ) return 0;
 #ifdef HAVE_XFT
-    return ColBuf[color].pixel;
+          return m_user_col_buf[col_pos].pixel;
 #else
-    return ColBuf[color];
+          return m_user_col_buf[col_pos];
 #endif
-  } else return 0;
+          break;
+      case AGUIXColor::SYSTEM_COLOR:
+          col_pos = color.getColor();
+          if ( col_pos >= m_system_colors ) col_pos = m_system_colors - 1;
+          if ( col_pos < 0 ) col_pos = 0;
+          if ( col_pos >= m_system_colors ) return 0;
+#ifdef HAVE_XFT
+          return m_system_col_buf[col_pos].pixel;
+#else
+          return m_system_col_buf[col_pos];
+#endif
+          break;
+    }
+    return 0;
 }
 
 #ifdef HAVE_XFT
-XftColor *AGUIX::getColBufEntry( int color )
+XftColor *AGUIX::getColBufEntry( const AGUIXColor &color )
 {
-  if ( color < colors && color >= 0 ) {
-    return &ColBuf[color];
-  } else return NULL;
+    switch ( color.getColorType() ) {
+      case AGUIXColor::USER_COLOR:
+          if ( color.getColor() < m_user_colors && color.getColor() >= 0 ) {
+              return &m_user_col_buf[color.getColor()];
+          }
+          break;
+      case AGUIXColor::SYSTEM_COLOR:
+          if ( color.getColor() < m_system_colors && color.getColor() >= 0 ) {
+              return &m_system_col_buf[color.getColor()];
+          }
+          break;
+    }
+    return NULL;
 }
 #else
-unsigned long AGUIX::getColBufEntry( int color )
+unsigned long AGUIX::getColBufEntry( const AGUIXColor &color )
 {
-  if ( color < colors && color >= 0 ) {
-    return ColBuf[color];
-  } else return 0;
+    switch ( color.getColorType() ) {
+      case AGUIXColor::USER_COLOR:
+          if ( color.getColor() < m_user_colors && color.getColor() >= 0 ) {
+              return m_user_col_buf[color.getColor()];
+          }
+          break;
+      case AGUIXColor::SYSTEM_COLOR:
+          if ( color.getColor() < m_system_colors && color.getColor() >= 0 ) {
+              return m_system_col_buf[color.getColor()];
+          }
+          break;
+    }
+    return 0;
 }
 #endif
 
@@ -663,6 +715,7 @@ Message *AGUIX::wait4mess( int mode )
       newmsg->mousex=LastEvent.xkey.x;
       newmsg->mousey=LastEvent.xkey.y;
       newmsg->keybuf = dupstring( keybuf );
+      newmsg->time = LastEvent.xkey.time;
       break;
     case ButtonPress:
     case ButtonRelease:
@@ -1067,14 +1120,9 @@ void AGUIX::ClearWin(Window win)
   XClearWindow(dsp,win);
 }
 
-void AGUIX::SetWindowBG(Window win,int color)
+void AGUIX::SetWindowBG( Window win, const AGUIXColor &color )
 {
-  if(color<0) color=0;
-  if(color<colors) {
-    XSetWindowBackground(dsp,win, getPixel( color ) );
-  } else {
-    XSetWindowBackground(dsp,win, getPixel( color - 1 ) );
-  }
+    XSetWindowBackground( dsp, win, getPixel( color ) );
 }
 
 void AGUIX::WindowtoBack(Window win)
@@ -1087,46 +1135,71 @@ void AGUIX::WindowtoFront(Window win)
   XRaiseWindow(dsp,win);
 }
 
-int AGUIX::changeColor(int index2,int red,int green,int blue)
+int AGUIX::changeColor( const AGUIXColor &index2, int red, int green, int blue )
 {
 #ifdef DEBUG
-  printf("changeColor:%d/%d\n",index2,colors);
+    if ( index2.getColorType() == AGUIXColor::SYSTEM_COLOR ) {
+        printf( "changeColor: system %d/%d\n", index2.getColor(), m_system_colors );
+    } else {
+        printf( "changeColor: user %d/%d\n", index2.getColor(), m_user_colors );
+    }
 #endif
-  if ( index2 < 0 || index2 >= colors ) return 1;
+    if ( index2.getColorType() == AGUIXColor::SYSTEM_COLOR ) {
+        if ( index2.getColor() < 0 || index2.getColor() >= m_system_colors ) return 1;
+    } else {
+        if ( index2.getColor() < 0 || index2.getColor() >= m_user_colors ) return 1;
+    }
 
 #ifdef HAVE_XFT
-  XftColorFree( dsp,
-                DefaultVisual( dsp, scr ),
-                cmap,
-                &ColBuf[index2] );
+    XftColor *buf_entry = NULL;
 
-  XRenderColor col;
-  col.red = red * 256;
-  col.green = green * 256;
-  col.blue = blue * 256;
-  col.alpha = 0xffff;
-  if ( ! XftColorAllocValue ( dsp,
-                              DefaultVisual( dsp, scr ),
-                              cmap,
-                              &col,
-                              &ColBuf[index2] ) ) {
-    return -1;
-  }
+    if ( index2.getColorType() == AGUIXColor::SYSTEM_COLOR ) {
+        buf_entry = &m_system_col_buf[index2.getColor()];
+    } else {
+        buf_entry = &m_user_col_buf[index2.getColor()];
+    }
+    XftColorFree( dsp,
+                  DefaultVisual( dsp, scr ),
+                  cmap,
+                  buf_entry );
+    
+    XRenderColor col;
+    col.red = red * 256;
+    col.green = green * 256;
+    col.blue = blue * 256;
+    col.alpha = 0xffff;
+    if ( ! XftColorAllocValue ( dsp,
+                                DefaultVisual( dsp, scr ),
+                                cmap,
+                                &col,
+                                buf_entry ) ) {
+        return -1;
+    }
 #else
-  XColor col;
-  XFreeColors(dsp,cmap,ColBuf+index2,1,0);
-  col.flags=DoRed|DoGreen|DoBlue;
-  col.red=red*256;
-  col.green=green*256;
-  col.blue=blue*256;
-  if(!XAllocColor(dsp,cmap,&col)) return -1;
-  ColBuf[index2]=col.pixel;
-#endif
+    XColor col;
+    unsigned long *buf_entry = NULL;
 
-  if ( index2 == 0 ) {
-    prepareExtColors( red, green, blue );
-  }
-  return 0;
+    if ( index2.getColorType() == AGUIXColor::SYSTEM_COLOR ) {
+        buf_entry = &m_system_col_buf[index2.getColor()];
+    } else {
+        buf_entry = &m_user_col_buf[index2.getColor()];
+    }
+
+    XFreeColors( dsp, cmap, buf_entry, 1, 0 );
+    col.flags = DoRed | DoGreen | DoBlue;
+    col.red = red * 256;
+    col.green = green * 256;
+    col.blue = blue * 256;
+    if ( !XAllocColor( dsp, cmap, &col ) ) return -1;
+    *buf_entry = col.pixel;
+#endif
+    
+    m_color_values[index2] = col_values_t( red, green, blue );
+
+    if ( index2.getColorType() == AGUIXColor::USER_COLOR ) {
+        updateSystemColors( index2.getColor() );
+    }
+    return 0;
 }
 
 AGUIXFont *AGUIX::getFont( const char *name )
@@ -1894,7 +1967,7 @@ void AGUIX::DrawDottedRectangle( Drawable buffer, int x, int y, int w, int h )
   DrawDottedLine( buffer, x, y + h - 1, x, y );
 }
 
-void AGUIX::setDottedFG( int color )
+void AGUIX::setDottedFG( const AGUIXColor &color )
 {
   setFG( dotted_gc, color );
 }
@@ -1927,12 +2000,12 @@ void AGUIX::DrawDashDRectangle( Drawable buffer, int x, int y, int w, int h )
   DrawDashDLine( buffer, x, y + h - 1, x, y );
 }
 
-void AGUIX::setDashDFG( int color )
+void AGUIX::setDashDFG( const AGUIXColor &color )
 {
   setFG( dashdouble_gc, color );
 }
 
-void AGUIX::setDashDBG( int color )
+void AGUIX::setDashDBG( const AGUIXColor &color )
 {
   setBG( dashdouble_gc, color );
 }
@@ -2134,14 +2207,15 @@ const AWindow* AGUIX::getTransientWindow() const
 
 int AGUIX::addDefaultColors()
 {
-  if ( AddColor( 160, 160, 160 ) < 0 ) return -1;
-  if ( AddColor( 0, 0, 0 ) < 0 ) return -1;
-  if ( AddColor( 255, 255, 255 ) < 0 ) return -1;
-  if ( AddColor( 0, 85, 187) < 0 ) return -1;
-  if ( AddColor( 204, 34, 0 ) < 0 ) return -1;
-  if ( AddColor( 50, 180, 20 ) < 0 ) return -1;
-  if ( AddColor( 119, 0, 119 ) < 0 ) return -1;
-  return AddColor( 238, 170, 68 );
+    if ( AddColor( 160, 160, 160 ).getColor() < 0 ) return -1;
+    if ( AddColor( 0, 0, 0 ).getColor() < 0 ) return -1;
+    if ( AddColor( 255, 255, 255 ).getColor() < 0 ) return -1;
+    if ( AddColor( 0, 85, 187).getColor() < 0 ) return -1;
+    if ( AddColor( 204, 34, 0 ).getColor() < 0 ) return -1;
+    if ( AddColor( 50, 180, 20 ).getColor() < 0 ) return -1;
+    if ( AddColor( 119, 0, 119 ).getColor() < 0 ) return -1;
+    if ( AddColor( 238, 170, 68 ).getColor() < 0 ) return -1;
+    return 0;
 }
 
 /*
@@ -2230,7 +2304,6 @@ int AGUIX::getTextWidth( const char *str, AGUIXFont *font )
 
 int AGUIX::getTextWidth( const char *str, AGUIXFont *font, int len )
 {
-  int slen;
   AGUIXFont *use_font;
 
   if ( str == NULL ) return 0;
@@ -2242,9 +2315,13 @@ int AGUIX::getTextWidth( const char *str, AGUIXFont *font, int len )
   }
   if ( use_font == NULL ) return 0;
 
-  slen = strlen( str );
-  if ( ( len < 0 ) || ( len > slen ) )
-    len = slen;
+  if ( len < 0 ) {
+      len = strlen( str );
+  } else {
+      if ( AGUIXUtils::stringIsShorter( str, len ) == true ) {
+          len = strlen( str );
+      }
+  }
 
   return use_font->textWidth( str, len );
 }
@@ -2287,9 +2364,15 @@ int AGUIX::getStrlen4WidthMaxlen( const char *str, int maxlen,
   }
 
   leftlen = leftw = 0;
-  rightlen = strlen( str );
-  if ( maxlen < rightlen && maxlen >= 0 ) {
-      rightlen = maxlen;
+
+  if ( maxlen < 0 ) {
+      rightlen = strlen( str );
+  } else {
+      if ( AGUIXUtils::stringIsShorter( str, maxlen ) == true ) {
+          rightlen = strlen( str );
+      } else {
+          rightlen = maxlen;
+      }
   }
   
   /* check if length is too large, some X libs have problems with too large strings
@@ -2404,81 +2487,6 @@ void AGUIX::unclip( AGUIXFont *font, DrawableCont *dc )
   }
 }
 
-void AGUIX::setFG( GC tgc, extcolor_t col )
-{
-  if ( tgc == 0) setFG( col );
-  else {
-    unsigned long pixel = _extcolors[col];
-    XSetForeground( dsp, tgc, pixel );
-  }
-}
-
-void AGUIX::setFG( extcolor_t col )
-{
-  unsigned long pixel = _extcolors[col];
-  XSetForeground( dsp, gc, pixel );
-}
-
-void AGUIX::setBG( GC tgc, extcolor_t col )
-{
-  if ( tgc == 0) setFG( col );
-  else {
-    unsigned long pixel = _extcolors[col];
-    XSetBackground( dsp, tgc, pixel );
-  }
-}
-
-void AGUIX::setBG( extcolor_t col )
-{
-  unsigned long pixel = _extcolors[col];
-  XSetBackground( dsp, gc, pixel );
-}
-
-void AGUIX::prepareExtColors( int gray_r, int gray_g, int gray_b )
-{
-  XColor col;
-  Visual *vis;
-
-  //TODO auf XftColor umstellen
-  /* ich muesste alle noetigen Farben allokieren
-   * alles andere waere ein zu grosser Aufwand
-   * Dazu muss ich jedoch eine neue Datenstruktur
-   * benutzen, da 1. die map die gesamte XftColor
-   * Struktur kopieren muesste und 2. ich zu-
-   * saetzlich speichern muss, welche Farben
-   * tatsaehclich allokiert wurden
-   */
-  _extcolors[EXTCOLOR_BLACK] = black;
-  _extcolors[EXTCOLOR_WHITE] = white;
-  if ( colors > 0 ) {
-    _extcolors[EXTCOLOR_LIGHTGRAY] = getPixel( 0 );
-    _extcolors[EXTCOLOR_DARKGRAY] = getPixel( 0 );
-  } else {
-    _extcolors[EXTCOLOR_LIGHTGRAY] = white;
-    _extcolors[EXTCOLOR_DARKGRAY] = black;
-  }
-  
-  vis = DefaultVisual( dsp, scr);
-
-  if ( vis->c_class == TrueColor ) {
-    col.flags = DoRed | DoGreen | DoBlue;
-    col.red = gray_r * 256;
-    col.green = gray_g * 256;
-    col.blue = gray_b * 256;
-    if ( XAllocColor( dsp, cmap, &col ) ) {
-      _extcolors[EXTCOLOR_LIGHTGRAY] = col.pixel;
-    }
-    
-    col.flags = DoRed | DoGreen | DoBlue;
-    col.red = ( gray_r * 9 / 10 ) * 256;
-    col.green = ( gray_g * 9 / 10 ) * 256;
-    col.blue = ( gray_b * 9 / 10 ) * 256;
-    if ( XAllocColor( dsp, cmap, &col ) ) {
-      _extcolors[EXTCOLOR_DARKGRAY] = col.pixel;
-    }
-  }
-}
-
 void AGUIX::drawBorder( Drawable buffer, bool pressed, int x, int y, int w, int h, int topright_space )
 {
   drawBorder( buffer, gc, pressed, x, y, w, h, topright_space );
@@ -2547,30 +2555,10 @@ void AGUIX::drawBorder( Drawable buffer, GC usegc, bool pressed, int x, int y, i
   DrawLine( buffer, usegc, bp[0].x, bp[0].y, bp[1].x, bp[1].y );
   DrawLine( buffer, usegc, bp[1].x, bp[1].y, bp[2].x, bp[2].y );
 
-  if ( extColorsAvail() == true ) {
-    setFG( usegc, EXTCOLOR_DARKGRAY );
-    DrawLine( buffer, usegc, gp[0].x, gp[0].y, gp[1].x, gp[1].y );
-    DrawLine( buffer, usegc, gp[1].x, gp[1].y, gp[2].x, gp[2].y );
-  }
-}
-
-bool AGUIX::extColorsAvail()
-{
-  Visual *vis;
-  vis = DefaultVisual( dsp, scr);
-
-  if ( vis->c_class == TrueColor ) {
-    return true;
-  } else {
-    return false;
-  }
-}
-
-void AGUIX::freeExtColors()
-{
-  //TODO nothing to do currently
-  // as I only support extended colors in true color
-  // mode where there are now color cells
+  // dark grey
+  setFG( usegc, AGUIXColor( 1, AGUIXColor::SYSTEM_COLOR ) );
+  DrawLine( buffer, usegc, gp[0].x, gp[0].y, gp[1].x, gp[1].y );
+  DrawLine( buffer, usegc, gp[1].x, gp[1].y, gp[2].x, gp[2].y );
 }
 
 Atom AGUIX::getAtom( atom_name_t name ) const
@@ -2669,12 +2657,12 @@ void AGUIX::xSync()
     XSync( dsp, False );
 }
 
-void AGUIX::popup_opened()
+void AGUIX::popupOpened()
 {
     m_open_popup_counter++;
 }
 
-void AGUIX::popup_closed()
+void AGUIX::popupClosed()
 {
     m_open_popup_counter--;
     if ( m_open_popup_counter < 0 ) {
@@ -2702,3 +2690,57 @@ int AGUIX::getWidgetRootPosition( Widget *widget, int *x, int *y )
     
     return 0;
 }
+
+void AGUIX::updateSystemColors( int changed_user_color )
+{
+    if ( changed_user_color == 0 || changed_user_color == 1 ) {
+        if ( m_color_values.count( AGUIXColor( 0, AGUIXColor::USER_COLOR ) ) > 0 ) {
+            int r1 = m_color_values[AGUIXColor( 0, AGUIXColor::USER_COLOR )].red,
+                g1 = m_color_values[AGUIXColor( 0, AGUIXColor::USER_COLOR )].green,
+                b1 = m_color_values[AGUIXColor( 0, AGUIXColor::USER_COLOR )].blue;
+            
+            int r2 = 0, g2 = 0, b2 = 0;
+
+            if ( m_color_values.count( AGUIXColor( 1, AGUIXColor::USER_COLOR ) ) > 0 ) {
+                r2 = m_color_values[AGUIXColor( 1, AGUIXColor::USER_COLOR )].red,
+                g2 = m_color_values[AGUIXColor( 1, AGUIXColor::USER_COLOR )].green,
+                b2 = m_color_values[AGUIXColor( 1, AGUIXColor::USER_COLOR )].blue;
+            }
+
+            // let the first system color be 2/3 between user color 0 and
+            // user color 1 (if available); usually this is some darker gray
+            r2 = ( r1 * 2 + r2 ) / 3;
+            g2 = ( g1 * 2 + g2 ) / 3;
+            b2 = ( b1 * 2 + b2 ) / 3;
+
+            if ( m_system_colors < 1 ) {
+                AddColor( r2, g2, b2, AGUIXColor::SYSTEM_COLOR );
+            } else {
+                changeColor( AGUIXColor( 0, AGUIXColor::SYSTEM_COLOR ), r2, g2, b2 );
+            }
+
+            // let the second system color be 9/10 of user color 0
+            r2 = ( r1 * 9 ) / 10;
+            g2 = ( g1 * 9 ) / 10;
+            b2 = ( b1 * 9 ) / 10;
+
+            if ( m_system_colors < 2 ) {
+                AddColor( r2, g2, b2, AGUIXColor::SYSTEM_COLOR );
+            } else {
+                changeColor( AGUIXColor( 1, AGUIXColor::SYSTEM_COLOR ), r2, g2, b2 );
+            }
+        }
+    }
+}
+
+bool AGUIX::getLastTypedWindowEvent( Window win, int type, XEvent *return_event )
+{
+    if ( return_event == NULL ) return false;
+    
+    int count = 0;
+
+    while ( XCheckTypedWindowEvent( dsp, win, type, return_event ) ) count++;
+
+    if ( count == 0 ) return false;
+    return true;
+}
