@@ -1,6 +1,6 @@
 /* aguix.h
  * This file belongs to Worker, a filemanager for UNIX/X11.
- * Copyright (C) 2001-2007 Ralf Hoffmann.
+ * Copyright (C) 2001-2008 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifndef AGUIX_H
@@ -44,6 +44,7 @@
 #include <list>
 
 #include "aguixfont.hh"
+#include "aguixcolor.hh"
 
 class AWindow;
 class PopUpWindow;
@@ -66,17 +67,17 @@ public:
   Display *getDisplay() const;
   int getScreen() const;
   void panic(const char *);
-  void setFG(int color);
-  void setBG(int color);
-  void setFG(GC gc,int color);
-  void setBG(GC gc,int color);
+  void setFG( const AGUIXColor &color );
+  void setBG( const AGUIXColor &color );
+  void setFG( GC gc, const AGUIXColor &color );
+  void setBG( GC gc, const AGUIXColor &color );
   int getCharHeight() const;
   void FillRectangle(Drawable buffer,int x,int y,int w,int h);
-  void DrawText( class DrawableCont &dc, const char *text, int x, int y, int color );
+  void DrawText( class DrawableCont &dc, const char *text, int x, int y, const AGUIXColor &color );
   void DrawLine(Drawable buffer,int px1,int py1,int px2,int py2);
   void DrawPoint(Drawable buffer,int x,int y);
   void FillRectangle(Drawable buffer,GC gc,int x,int y,int w,int h);
-  void DrawText( class DrawableCont &dc, AGUIXFont *, const char *text, int x, int y, int color );
+  void DrawText( class DrawableCont &dc, AGUIXFont *, const char *text, int x, int y, const AGUIXColor &color );
   void DrawLine(Drawable buffer,GC gc,int px1,int py1,int px2,int py2);
   void DrawPoint(Drawable buffer,GC gc,int x,int y);
 
@@ -86,17 +87,17 @@ public:
   void unclip( AGUIXFont *font, class DrawableCont *dc );
 
   void ClearWin(Window win);
-  void SetWindowBG(Window win,int color);
+  void SetWindowBG( Window win, const AGUIXColor &color );
   int getFontBaseline() const;
-  int AddColor(int red,int green,int blue);
-  void removeLastColor();
-  int changeColor(int,int,int,int);
-  int getMaxCols() const;
-  unsigned long getPixel(int color) const;
+  AGUIXColor AddColor( int red, int green, int blue, AGUIXColor::color_type_t type = AGUIXColor::USER_COLOR );
+  void freeColors();
+  int changeColor( const AGUIXColor &index, int red, int green ,int blue );
+  int getNumberOfColorsForType( AGUIXColor::color_type_t type ) const;
+  unsigned long getPixel( const AGUIXColor &color ) const;
 #ifdef HAVE_XFT
-  XftColor *getColBufEntry( int color );
+  XftColor *getColBufEntry( const AGUIXColor &color );
 #else
-  unsigned long getColBufEntry( int color );
+  unsigned long getColBufEntry( const AGUIXColor &color );
 #endif
 
   std::string getClassname() const;
@@ -157,15 +158,15 @@ public:
 
   void DrawDottedLine( Drawable buffer, int px1, int py1, int px2, int py2 );
   void DrawDottedRectangle( Drawable buffer, int x, int y, int w, int h );
-  void setDottedFG( int color );
+  void setDottedFG( const AGUIXColor &color );
 
   void DrawDashXorLine( Drawable buffer, int px1, int py1, int px2, int py2 );
   void DrawDashXorRectangle( Drawable buffer, int x, int y, int w, int h );
   
   void DrawDashDLine( Drawable buffer, int px1, int py1, int px2, int py2 );
   void DrawDashDRectangle( Drawable buffer, int x, int y, int w, int h );
-  void setDashDFG( int color );
-  void setDashDBG( int color );
+  void setDashDFG( const AGUIXColor &color );
+  void setDashDBG( const AGUIXColor &color );
 
   static bool isModifier( KeySym key );
   static char *getStringForKeySym( KeySym key );
@@ -183,7 +184,6 @@ public:
 #endif
   void setTransientWindow( AWindow *twin = NULL );
   const AWindow* getTransientWindow() const;
-  int addDefaultColors();
   KeySym getLastKeyRelease();
   unsigned int getLastMouseRelease();
   void msgLock( Widget *e );
@@ -206,31 +206,27 @@ public:
 
   static long timerEvent;
 
-  typedef enum { EXTCOLOR_BLACK, EXTCOLOR_DARKGRAY, EXTCOLOR_LIGHTGRAY, EXTCOLOR_WHITE } extcolor_t;
-  void setFG( GC tgc, extcolor_t col );
-  void setFG( extcolor_t col );
-  void setBG( GC tgc, extcolor_t col );
-  void setBG( extcolor_t col );
-
   void drawBorder( Drawable buffer, GC usegc, bool pressed, int x, int y, int w, int h, int topright_space );
   void drawBorder( Drawable buffer, bool pressed, int x, int y, int w, int h, int topright_space );
-  bool extColorsAvail();
 
   typedef enum { CLOSE_ATOM, NET_WM_NAME_ATOM, NET_WM_ICON_NAME_ATOM } atom_name_t;
   Atom getAtom( atom_name_t name ) const;
 
+  //TODO also convert to AGUIXColor
   static void setColorForName( const std::string &name, int col );
   static int getColorForName( const std::string &name );
 
   void registerPopUpWindow( PopUpWindow *win );
   void hidePopUpWindows( int group_id = -1 );
   void hideOtherPopUpWindows( int group_id = -1 );
-  void popup_opened();
-  void popup_closed();
+  void popupOpened();
+  void popupClosed();
 
   void xSync();
   
   int getWidgetRootPosition( Widget *widget, int *x, int *y );
+  bool getLastTypedWindowEvent( Window win, int type, XEvent *return_event );
+  int addDefaultColors();
 private:
   Display *dsp;
   int scr;
@@ -247,11 +243,27 @@ private:
   int initOK;
   int CharHeight;
 #ifdef HAVE_XFT
-  XftColor ColBuf[256];
+  XftColor m_user_col_buf[256];
+  XftColor m_system_col_buf[16];
 #else
-  unsigned long ColBuf[256];
+  unsigned long m_user_col_buf[256];
+  unsigned long m_system_col_buf[16];
 #endif
-  int colors;
+  int m_user_colors, m_system_colors;
+
+  typedef struct _col_values_t {
+      _col_values_t() : red( 0 ),
+                        green( 0 ),
+                        blue( 0 )
+      {}
+      _col_values_t( int r, int g, int b) : red( r ),
+                                            green( g ),
+                                            blue( b )
+      {}
+      int red, green, blue;
+  } col_values_t;
+  std::map< AGUIXColor, col_values_t, AGUIXColor::compare_func > m_color_values;
+
   XEvent LastEvent;
   std::list<AWindow*> wins;
   typedef std::list<AWindow*>::const_iterator wins_cit_t;
@@ -278,6 +290,7 @@ private:
   
   bool privatecmap;
   void changeColormap();
+  void updateSystemColors( int changed_user_color );
 
   Cursor cursors[MAXCURSORS];
   GUIElement *cutstart,*pastestart;
@@ -308,10 +321,6 @@ private:
   enum { TIMER_TICKS = 25 }; //HARDCODED
   long lastTimerEventNr;
 
-  std::map<extcolor_t,unsigned long> _extcolors;
-  void prepareExtColors( int gray_r, int gray_g, int gray_b );
-  void freeExtColors();
-
   static std::map<std::string,int> _colors_for_widgets;
 
   void checkPopUpWindows( Message * );
