@@ -149,6 +149,8 @@ FieldListView::FieldListView( AGUIX *caguix, int cx, int cy, int cwidth, int che
   _last_single_middle_time = 0;
 
   m_complete_update = false;
+
+  gettimeofday( &m_last_update_time, NULL );
 }
 
 FieldListView::~FieldListView()
@@ -2237,15 +2239,15 @@ void FieldListView::runSelectHandler( int p )
   if ( selecthandler != NULL ) ( selecthandler )( this, p );
 }
 
-void FieldListView::showRow( int row )
+void FieldListView::showRow( int row, bool redraw )
 {
   if ( ( row >=0 ) && ( row < elements ) ){
     if ( row < yoffset ) {
-      if ( abs( row - yoffset ) == 1 ) scrollV1( -1 );
-      else scrollV( row - yoffset );
+      if ( abs( row - yoffset ) == 1 ) scrollV1( -1, ( redraw == true ) ? false : true );
+      else scrollV( row - yoffset, ( redraw == true ) ? false : true );
     } else if ( ( row - yoffset ) >= maxdisplayv ) {
-      if ( abs( row - yoffset - maxdisplayv + 1 ) == 1 ) scrollV1( 1 );
-      else scrollV( row - yoffset - maxdisplayv + 1 );
+      if ( abs( row - yoffset - maxdisplayv + 1 ) == 1 ) scrollV1( 1, ( redraw == true ) ? false : true );
+      else scrollV( row - yoffset - maxdisplayv + 1, ( redraw == true ) ? false : true );
     }
   }
 }
@@ -3760,3 +3762,21 @@ void FieldListView::simpleRedrawUpdatedRows()
         m_updated_rows.clear();
     }
 }
+
+// just a little helper function to limit redraws to approx. 25 fps
+// for applications with a lot of updates
+void FieldListView::timeLimitedRedraw()
+{
+    struct timeval now;
+    double diff;
+    
+    gettimeofday( &now, NULL );
+    diff = now.tv_sec + now.tv_usec / 1000000.0;
+    diff -= m_last_update_time.tv_sec + m_last_update_time.tv_usec / 1000000.0;
+
+    // hard coded FPS limit
+    if ( diff >= ( 1.0 / 25.0 ) ) {
+        redraw();
+        m_last_update_time = now;
+    }
+}
