@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 /* $Id: message.cc,v 1.8 2004/10/27 20:40:48 ralf Exp $ */
 
@@ -96,6 +96,7 @@ AGMessage *AGUIX_allocAGMessage( Message *msg )
       agmsg->key.key = msg->key;
       agmsg->key.keystate = msg->keystate;
       agmsg->key.keybuf = dupstring( msg->keybuf );
+      agmsg->key.time = msg->time;
       break;
     case ButtonPress:
       agmsg->type = AG_MOUSEPRESSED;
