@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifndef MESSAGE_H
@@ -79,6 +79,7 @@ typedef struct agkey {
   KeySym key;
   unsigned int keystate;
   char *keybuf;
+  Time time;
 } AGKey;
 
 typedef AGKey AGKeyPressed;
