@@ -1,6 +1,6 @@
 /* popupmenu.cc
  * This file belongs to Worker, a filemanager for UNIX/X11.
- * Copyright (C) 2007 Ralf Hoffmann.
+ * Copyright (C) 2007-2008 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -16,13 +16,15 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include <iostream>
 #include <stdlib.h>
 #include "popupmenu.hh"
 #include "drawablecont.hh"
+#include "utf8.hh"
+#include <stdexcept>
 
 PopUpMenu::PopUpMenu( AGUIX *parent,
                       const std::list<PopUpEntry> &entries,
@@ -42,7 +44,8 @@ PopUpMenu::PopUpMenu( AGUIX *parent,
                                        m_grabbed_keyboard( -1 ),
                                        m_grabbed_pointer( -1 ),
                                        m_border_width( 2 ),
-                                       m_ignore_button_release( true )
+                                       m_ignore_button_release( true ),
+                                       m_infix_filter( "" )
 {
     std::list<PopUpEntry>::const_iterator it1;
 
@@ -191,6 +194,8 @@ void PopUpMenu::hide()
         agmsg->type = AG_POPUPMENU_CLOSED;
         _aguix->putAGMsg( agmsg );
     }
+    
+    m_infix_filter = "";
 }
 
 int PopUpMenu::create()
@@ -208,7 +213,13 @@ int PopUpMenu::create()
     XWindowAttributes curattr;
     
     XGetWindowAttributes( _aguix->getDisplay(), win, &curattr );
-    XSelectInput( _aguix->getDisplay(), win, curattr.your_event_mask | PointerMotionMask | EnterWindowMask | LeaveWindowMask );
+    XSelectInput( _aguix->getDisplay(), win,
+                  curattr.your_event_mask |
+                  PointerMotionMask |
+                  EnterWindowMask |
+                  LeaveWindowMask |
+                  FocusChangeMask |
+                  StructureNotifyMask );
     
     setDimensions();
 
@@ -276,7 +287,7 @@ void PopUpMenu::showEntry( int entry )
                 it1->submenu->hide();
             }
         } else {
-            if ( it1->type == SUBMENU ) {
+            if ( it1->type == SUBMENU && isVisibleEntry( entry ) == true ) {
                 if ( it1->submenu->isVisible() == false ) {
                     int sub_y, sub_x;
                     
@@ -395,6 +406,17 @@ bool PopUpMenu::handleMessage( XEvent *e,Message *msg )
                 }
             } else if ( msg->key == XK_Escape ) {
                 _aguix->hidePopUpWindows( getGroupID() );
+            } else if ( msg->key == XK_BackSpace ) {
+                //remove last character (think of utf8)
+                //removeLastInfixChar();
+                
+                // currently I just clear the filter, I think
+                // this is better for daily use
+                m_infix_filter = "";
+                updateInfixFilter();
+            } else if ( strlen( msg->keybuf ) > 0 ) {
+                //append to infix
+                addStringToFilter( msg->keybuf );
             }
             ret_val = true;
         }
@@ -422,11 +444,13 @@ bool PopUpMenu::handleMessage( XEvent *e,Message *msg )
                    abs( mx - m_last_motion_x ) < 1 ) {
                   entry = getEntryForYPos( my );
                   if ( entry >= 0 && entry < (int)m_entries.size() ) {
-                      if ( m_highlighted_entry != entry ) {
+                      if ( m_highlighted_entry != entry && isVisibleEntry( entry ) == true ) {
                           m_open_after_timeout = true;
                           m_highlighted_entry = entry;
                           m_counter = 0;
                           
+                          changeFocusTo( this );
+                          
                           enableTimer();
                           redraw();
                       }
@@ -443,11 +467,13 @@ bool PopUpMenu::handleMessage( XEvent *e,Message *msg )
               _aguix->queryPointer( win, &mx, &my );
               entry = getEntryForYPos( my );
               if ( entry >= 0 && entry < (int)m_entries.size() ) {
-                  m_highlighted_entry = entry;
+                  if ( isVisibleEntry( entry ) == true ) {
+                      m_highlighted_entry = entry;
 
-                  disableTimer();
-                  showEntry( entry );
-                  redraw();
+                      disableTimer();
+                      showEntry( entry );
+                      redraw();
+                  }
               }
               ret_val = true;
               m_ignore_button_release = false;
@@ -457,14 +483,20 @@ bool PopUpMenu::handleMessage( XEvent *e,Message *msg )
                   _aguix->queryPointer( win, &mx, &my );
                   entry = getEntryForYPos( my );
                   if ( entry >= 0 && entry < (int)m_entries.size() ) {
-                      m_highlighted_entry = entry;
+                      if ( isVisibleEntry( entry ) == true ) {
+                          m_highlighted_entry = entry;
                       
-                      disableTimer();
-                      activateEntry( m_highlighted_entry );
+                          disableTimer();
+                          activateEntry( m_highlighted_entry );
+                      }
                   }
               }
               ret_val = true;
               break;
+          case UnmapNotify:
+          case FocusOut:
+              _aguix->hidePopUpWindows( getGroupID() );
+              break;
           default:
               break;
         }
@@ -514,7 +546,7 @@ void PopUpMenu::activateEntry( int entry )
     for ( std::vector<InternPopUpEntry>::iterator it1 = m_entries.begin();
           it1 != m_entries.end();
           it1++, ty++ ) {
-        if ( ty == entry ) {
+        if ( ty == entry && isVisibleEntry( entry ) == true ) {
             if ( it1->type == SUBMENU ) {
                 showEntry( m_highlighted_entry );
             } else if ( it1->type == NORMAL ) {
@@ -536,6 +568,7 @@ void PopUpMenu::setHighlightEntry( int entry )
         if ( m_highlighted_entry != entry ) {
             m_open_after_timeout = true;
             m_highlighted_entry = entry;
+            m_highlighted_entry = forwardToVisibleEntry( m_highlighted_entry, 1 );
             m_counter = 0;
             
             enableTimer();
@@ -597,10 +630,21 @@ void PopUpMenu::redrawEntry( int entry, int top_y )
                                         getWidth() - 2 * getBorderWidth(),
                                         getEntryHeight( entry ), 0 );
                 }
-                _aguix->DrawText( dc, m_entries[entry].name.c_str(),
-                                  getBorderWidth() + 1,
-                                  top_y + 1,
-                                  ( m_highlighted_entry == entry ) ? 2 : 1 );
+                if ( isVisibleEntry( entry ) == true ) {
+                    _aguix->DrawText( dc, m_entries[entry].name.c_str(),
+                                      getBorderWidth() + 1,
+                                      top_y + 1,
+                                      ( m_highlighted_entry == entry ) ? 2 : 1 );
+                } else {
+                    _aguix->DrawText( dc, m_entries[entry].name.c_str(),
+                                      getBorderWidth() + 1 + 1,
+                                      top_y + 1 + 1,
+                                      2 );
+                    _aguix->DrawText( dc, m_entries[entry].name.c_str(),
+                                      getBorderWidth() + 1,
+                                      top_y + 1,
+                                      AGUIXColor( 0, AGUIXColor::SYSTEM_COLOR ) );
+                }
                                     
                 if ( m_entries[entry].type == SUBMENU ) {
                     int submenu_char_w = _aguix->getCharHeight();
@@ -695,7 +739,7 @@ int PopUpMenu::forwardToVisibleEntry( int pos, int direction )
     old_pos = pos;
     
     do {
-        if ( m_entries[pos].type != HLINE && m_entries[pos].type != HEADER )
+        if ( isVisibleEntry( pos ) && m_entries[pos].type != HLINE && m_entries[pos].type != HEADER )
             break;
         
         if ( direction < 0 ) {
@@ -707,5 +751,118 @@ int PopUpMenu::forwardToVisibleEntry( int pos, int direction )
         }
     } while ( pos != old_pos );
     
+    if ( isVisibleEntry( pos ) == false ) return -1;
     return pos;
 }
+
+void PopUpMenu::updateInfixFilter()
+{
+    //move highlight to next visible
+    m_highlighted_entry = forwardToVisibleEntry( m_highlighted_entry, 1 );
+
+    m_open_after_timeout = true;
+    m_counter = 0;
+    enableTimer();
+    //redraw menu
+    redraw();
+}
+
+bool PopUpMenu::isVisibleEntry( int pos ) const
+{
+    if ( pos >= 0 && pos < (int)m_entries.size() ) {
+        std::string lower_infix = AGUIXUtils::tolower( m_infix_filter );
+        std::string lower_name = AGUIXUtils::tolower( m_entries[pos].name );
+        
+        if ( lower_name.find( lower_infix ) == std::string::npos ) return false;
+        return true;
+    }
+    return false;
+}
+
+bool PopUpMenu::checkForVisibleEntries( const std::string &filter )
+{
+    std::string lower_filter = AGUIXUtils::tolower( filter );
+    
+    for ( std::vector<InternPopUpEntry>::iterator it1 = m_entries.begin();
+          it1 != m_entries.end();
+          it1++ ) {
+        std::string lower_name;
+        
+        switch ( it1->type ) {
+          case NORMAL:
+          case SUBMENU:
+            lower_name = AGUIXUtils::tolower( it1->name );
+            if ( lower_name.find( lower_filter ) != std::string::npos ) {
+                return true;
+            }
+            break;
+          default:
+            break;
+        }
+    }
+    
+    return false;
+}
+
+void PopUpMenu::addStringToFilter( const char *str )
+{
+    std::string temp_filter = m_infix_filter;
+    temp_filter += str;
+    
+    if ( checkForVisibleEntries( temp_filter ) == true ) {
+        m_infix_filter = temp_filter;
+        updateInfixFilter();
+    }
+}
+
+void PopUpMenu::removeLastInfixChar()
+{
+    int prev_char = m_infix_filter.length();
+    UTF8::movePosToPrevChar( m_infix_filter.c_str(), prev_char );
+
+    if ( prev_char < 0 ) {
+        m_infix_filter = "";
+    } else {
+        try {
+            m_infix_filter.erase( prev_char );
+        } catch ( std::out_of_range ) {
+            m_infix_filter = "";
+        }
+    }
+    updateInfixFilter();
+}
+
+void PopUpMenu::changeFocusTo( PopUpMenu *owner, change_focus_run_t mode )
+{
+    if ( owner == this && m_keyboard_focus == true ) return;
+
+    if ( mode == BOTTOM_UP_RUN ) {
+        if ( m_parent_menu != NULL ) {
+            m_parent_menu->changeFocusTo( owner, mode );
+        } else {
+            changeFocusTo( owner, TOP_DOWN_RUN );
+        }
+    } else if ( mode == TOP_DOWN_RUN ) {
+        if ( owner != this ) {
+            if ( m_keyboard_focus == true ) {
+                setKeyboardFocus( false );
+            }
+        }
+        
+        for ( std::vector<InternPopUpEntry>::iterator it1 = m_entries.begin();
+              it1 != m_entries.end();
+              it1++ ) {
+            switch ( it1->type ) {
+              case SUBMENU:
+                  if ( it1->submenu != NULL ) it1->submenu->changeFocusTo( owner, mode );
+                  break;
+              default:
+                  break;
+            }
+        }
+        
+        if ( owner == this ) {
+            setKeyboardFocus( true );
+        }
+    }
+}
