@@ -1,6 +1,6 @@
 /* textstorage.cc
  * This file belongs to Worker, a filemanager for UNIX/X11.
- * Copyright (C) 2005-2006 Ralf Hoffmann.
+ * Copyright (C) 2005-2008 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -16,23 +16,23 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "textstorage.h"
 #include "utf8.hh"
 
-TextStorageString::TextStorageString( const char *buf, int len, AWidth &lencalc ) : _lencalc( lencalc ),
-                                                                                    m_current_line_limit( -1 )
+TextStorageString::TextStorageString( const char *buf, int len, const RefCount<AWidth> &lencalc, int line_limit ) : m_lencalc( lencalc ),
+                                                                                                                    m_current_line_limit( -1 )
 {
   std::string s1( buf, len );
-  initBuffer( s1 );
+  initBuffer( s1, line_limit );
 }
 
-TextStorageString::TextStorageString( std::string content, AWidth &lencalc ) : _lencalc( lencalc ),
-                                                                               m_current_line_limit( -1 )
+TextStorageString::TextStorageString( const std::string &content, const RefCount<AWidth> &lencalc, int line_limit ) : m_lencalc( lencalc ),
+                                                                                                                      m_current_line_limit( -1 )
 {
-  initBuffer( content );
+    initBuffer( content, line_limit );
 }
 
 int TextStorageString::getNrOfLines() const
@@ -78,7 +78,7 @@ int TextStorageString::getLine( int line_nr, unsigned int offset, std::string &r
   return getLine( line_nr, offset, -1, return_line );
 }
 
-void TextStorageString::initBuffer( std::string &content, int line_limit_width )
+void TextStorageString::initBuffer( const std::string &content, int line_limit_width )
 {
   unsigned int i;
   int current_line = 0, current_subline = 0;
@@ -90,6 +90,9 @@ void TextStorageString::initBuffer( std::string &content, int line_limit_width )
   _real_lines.push_back( std::pair<int,int>( current_line, current_subline ) );
   m_line_width_cached.clear();
 
+  m_unwrapped_line_offsets.clear();
+  m_unwrapped_line_offsets.push_back( 0 );
+
   if ( line_limit_width < 1 ) {
     // the easiest stuff: no line limit
     for ( i = 0; i < buffer.length(); i++ ) {
@@ -99,6 +102,7 @@ void TextStorageString::initBuffer( std::string &content, int line_limit_width )
 	line_offsets.push_back( i + 1 );  // if newline is last character
                                           // this line offset will be outside
                                           // the buffer
+        m_unwrapped_line_offsets.push_back( i + 1 );
         _real_lines.push_back( std::pair<int,int>( current_line, current_subline ) );
       }
     }
@@ -127,7 +131,7 @@ void TextStorageString::initBuffer( std::string &content, int line_limit_width )
         if ( token_characters < 1 )
           token_characters = 1;
         
-	token_width = _lencalc.getWidth( buffer.c_str() + i, token_characters );
+	token_width = m_lencalc->getWidth( buffer.c_str() + i, token_characters );
 
 	if ( ( cur_line_width + token_width ) <= (unsigned int)line_limit_width ) {
 	  cur_line_width += token_width;
@@ -135,8 +139,8 @@ void TextStorageString::initBuffer( std::string &content, int line_limit_width )
 	} else if ( cur_line_width == 0 ) {
 	  // no character in current line and no space for whole token
 	  // so split it
-          token_characters = _lencalc.getStrlen4WidthMaxlen( buffer.c_str() + i, token_characters,
-                                                             line_limit_width, NULL );
+          token_characters = m_lencalc->getStrlen4WidthMaxlen( buffer.c_str() + i, token_characters,
+                                                               line_limit_width, NULL );
 
 	  if ( token_characters < 1 ) {
 	    // no room for at least one character at line start so we need to take
@@ -163,6 +167,10 @@ void TextStorageString::initBuffer( std::string &content, int line_limit_width )
 	cur_line_width = 0;
 
         _real_lines.push_back( std::pair<int,int>( current_line, current_subline ) );
+
+        if ( current_subline == 0 ) {
+            m_unwrapped_line_offsets.push_back( i );
+        }
       }
     }
   }
@@ -170,16 +178,6 @@ void TextStorageString::initBuffer( std::string &content, int line_limit_width )
   m_current_line_limit = line_limit_width;
 }
 
-TextStorageString &TextStorageString::operator=( const TextStorageString &other )
-{
-  if ( this != &other ) {
-    buffer = other.buffer;
-    line_offsets = other.line_offsets;
-    _real_lines = other._real_lines;
-  }
-  return *this;
-}
-
 int TextStorageString::getMaxLineWidth() const
 {
   int max_line_len = 0, l, i;
@@ -219,7 +217,7 @@ int TextStorageString::getLineWidth( int line_nr ) const
           // no next line
           line_len = buffer.length() - start_offset;
       }
-      width = _lencalc.getWidth( buffer.c_str() + start_offset, line_len );
+      width = m_lencalc->getWidth( buffer.c_str() + start_offset, line_len );
 
       if ( (int)m_line_width_cached.size() <= line_nr ) {
           m_line_width_cached.resize( line_nr + 1, -1 );
@@ -258,6 +256,11 @@ void TextStorageString::setLineLimit( int new_limit )
     initBuffer( buffer, new_limit );
 }
 
+int TextStorageString::getLineLimit()
+{
+    return m_current_line_limit;
+}
+
 int TextStorageString::getLineCharacters( int line_nr ) const
 {
   unsigned int start_offset, line_len;  // remember offset from line_offsets are not necessarly inside buffer
@@ -290,7 +293,7 @@ std::pair<int,int> TextStorageString::getRealLinePair( int line_nr ) const
 
 AWidth &TextStorageString::getAWidth()
 {
-  return _lencalc;
+  return *m_lencalc;
 }
 
 int TextStorageString::findLineNr( std::pair<int,int> real_line ) const
@@ -306,3 +309,103 @@ int TextStorageString::findLineNr( std::pair<int,int> real_line ) const
   if ( l >= s ) return 0;
   return l;
 }
+
+int TextStorageString::getNrOfUnwrappedLines() const
+{
+    return m_unwrapped_line_offsets.size();
+}
+
+int TextStorageString::getUnwrappedLine( int line_nr,
+                                         unsigned int offset,
+                                         int len,
+                                         std::string &return_line ) const
+{
+    unsigned int start_offset, line_len;  // remember offset from line_offsets are not necessarly inside buffer
+    
+    if ( line_nr < 0 || line_nr >= getNrOfUnwrappedLines() ) return 1;
+    
+    start_offset = m_unwrapped_line_offsets[ line_nr ];
+    line_len = getUnwrappedLineCharacters( line_nr );
+    
+    return_line = "";
+    
+    if ( offset < line_len ) {
+        // offset given still inside line
+        
+        if ( UTF8::isValidCharacter( buffer.c_str() + start_offset + offset ) == false ) {
+            int n = (int)offset;
+            UTF8::movePosToPrevChar( buffer.c_str() + start_offset, n );
+            offset = (unsigned int )n;
+        }
+        
+        start_offset += offset;
+        line_len -= offset;
+        
+        if ( len >= 0 ) {
+            // line limiter given
+            if ( (unsigned int )len < line_len ) line_len = (unsigned int)len;
+        }
+        if ( ( start_offset < buffer.length() ) && ( line_len > 0 ) )
+            return_line.append( buffer, start_offset, line_len );
+    }
+    return 0;
+}
+
+int TextStorageString::getUnwrappedLine( int line_nr,
+                                         unsigned int offset,
+                                         std::string &return_line ) const
+{
+    return getUnwrappedLine( line_nr, offset, -1, return_line );
+}
+
+int TextStorageString::getUnwrappedLineCharacters( int line_nr ) const
+{
+    unsigned int start_offset, line_len;  // remember offset from line_offsets are not necessarly inside buffer
+    
+    if ( line_nr < 0 || line_nr >= getNrOfUnwrappedLines() ) return -1;
+    
+    start_offset = m_unwrapped_line_offsets[ line_nr ];
+    if ( ( line_nr + 1 ) < getNrOfUnwrappedLines() ) {
+        // next line available
+        line_len = m_unwrapped_line_offsets[ line_nr + 1 ] - start_offset;
+        
+        return line_len - 1; // minus 1 for backslash n
+    } else {
+        // no next line
+        return buffer.length() - start_offset;
+    }
+}
+
+std::pair< std::pair< int, int >, int > TextStorageString::getLineForOffset( int unwrapped_line_nr, int line_offset ) const
+{
+    if ( unwrapped_line_nr < 0 || unwrapped_line_nr >= getNrOfUnwrappedLines() ) {
+        return std::pair< std::pair< int, int >, int>( std::pair<int, int>( -1, -1 ), -1 );
+    }
+
+    if ( m_current_line_limit < 1 ) {
+        // no wrapping so it's just the original offset in line (unwrapped_line_nr,0)
+        int l = getLineCharacters( unwrapped_line_nr );
+        if ( line_offset <= l ) {
+            return std::pair< std::pair< int, int >, int>( std::pair<int, int>( unwrapped_line_nr, 0 ), line_offset );
+        } else {
+            return std::pair< std::pair< int, int >, int>( std::pair<int, int>( -1, -1 ), -1 );
+        }
+    }
+
+    int line_nr;
+    int current_line_offset = 0;
+
+    line_nr = findLineNr( std::pair<int,int>( unwrapped_line_nr, 0 ) );
+    while ( line_nr < (int)_real_lines.size() && _real_lines[line_nr].first == unwrapped_line_nr ) {
+        int l = getLineCharacters( line_nr );
+
+        if ( line_offset >= current_line_offset && line_offset < ( current_line_offset + l ) ) break;
+        line_nr++;
+        current_line_offset += l;
+    }
+    if ( line_nr < (int)_real_lines.size() && _real_lines[line_nr].first == unwrapped_line_nr ) {
+        return std::pair< std::pair< int, int >, int>( _real_lines[line_nr], line_offset - current_line_offset );
+    } else {
+        return std::pair< std::pair< int, int >, int>( std::pair<int, int>( -1, -1 ), -1 );
+    }
+}
