@@ -1,6 +1,6 @@
 /* util.cc
  * This file belongs to Worker, a filemanager for UNIX/X11.
- * Copyright (C) 2001-2007 Ralf Hoffmann.
+ * Copyright (C) 2001-2008 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -16,11 +16,15 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "util.h"
 #include "lowlevelfunc.h"
+#include <cctype>
+#include <algorithm>
+#include <stdarg.h>
+#include <functional>
 
 int createLines(const char *text,char ***ret_lines)
 {
@@ -102,4 +106,61 @@ namespace AGUIXUtils
         return return_str;
     }
     
+    template <class T>
+    struct my_tolower_func : public std::unary_function< T, T > 
+    {
+        T operator() ( const T& c ) const 
+        {
+            return std::tolower( c );
+        }
+    };
+
+    std::string tolower( const std::string &str )
+    {
+        std::string res( str );
+        
+        std::transform( res.begin(),
+                        res.end(),
+                        res.begin(),
+                        my_tolower_func<char>() );
+        return res;
+    }
+
+    bool stringIsShorter( const char *str, int len )
+    {
+        if ( str == NULL ) return true;
+        
+        int my_len;
+        
+        for ( my_len = 0; str[my_len] != '\0' && my_len < len; my_len++ );
+        
+        if ( my_len < len ) return true;
+        return false;
+    }
+
+    std::string formatStringToString( const char *format, ... )
+    {
+        va_list va;
+        int n, size = 1024;
+        char *tstr;
+        std::string string1;
+        
+        if ( format != NULL ) {
+            // first create str
+            while ( 1 ) {
+                tstr = (char*)_allocsafe( size );
+                va_start( va, format );
+                n = vsnprintf( tstr, size, format, va );
+                va_end( va );
+                if ( n > -1 && n < size ) break;
+                _freesafe( tstr );
+                size *= 2;
+            }
+            string1 = tstr;
+            _freesafe( tstr );
+            return string1;
+        }
+        return "";
+    }
+
 }
