@@ -43,6 +43,14 @@ struct my_tolower : public std::unary_function<T,T>
     }
 };
 
+struct my_sort : public std::binary_function< BookmarkDBEntry, BookmarkDBEntry, bool > 
+{
+    bool operator() ( const BookmarkDBEntry& c1, const BookmarkDBEntry& c2 ) const 
+    {
+        return c1.getName() < c2.getName();
+    }
+};
+
 std::string tolower( const std::string &str )
 {
     std::string res( str );
@@ -73,6 +81,9 @@ std::list< BookmarkDBEntry > BookmarkDBFilter::getEntries( const std::string &ca
             }
         }
     }
+
+    res.sort( my_sort() );
+
     return res;
 }
 
