@@ -343,8 +343,9 @@
      XFTFONTS_WCP = 569,
      USEMAGIC_WCP = 570,
      MAGICPATTERN_WCP = 571,
-     STRING_WCP = 572,
-     NUM_WCP = 573
+     MAGICCOMPRESSED_WCP = 572,
+     STRING_WCP = 573,
+     NUM_WCP = 574
    };
 #endif
 /* Tokens.  */
@@ -662,8 +663,9 @@
 #define XFTFONTS_WCP 569
 #define USEMAGIC_WCP 570
 #define MAGICPATTERN_WCP 571
-#define STRING_WCP 572
-#define NUM_WCP 573
+#define MAGICCOMPRESSED_WCP 572
+#define STRING_WCP 573
+#define NUM_WCP 574
 
 
 
@@ -675,7 +677,7 @@ typedef union YYSTYPE {
   char *strptr;
 } YYSTYPE;
 /* Line 1447 of yacc.c.  */
-#line 679 "configparser.tab.h"
+#line 681 "configparser.tab.h"
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
