@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "datei.h"
@@ -1264,7 +1264,7 @@ int Datei::getCharacter()
   
   if ( fd < 0 ) return EOF;
   
-  count = worker_read( fd, cf, 1 );
+  count = ::worker_read( fd, cf, 1 );
   
   if ( count < 1 ) {
     iseof = true;
@@ -1320,7 +1320,7 @@ int Datei::copyfile( const char *dest, const char *source )
   writebytes = -1;
   erg = 0;
   do {
-    readbytes = worker_read( fdr, buf, BUFSIZE );
+    readbytes = ::worker_read( fdr, buf, BUFSIZE );
     if ( readbytes < 0 ) {
       // error while reading
       erg = 1;
@@ -1523,3 +1523,18 @@ void Datei::configCloseSection()
   str1 += "}";
   putLine( str1.c_str() );
 }
+
+size_t Datei::read( char *buffer, size_t len )
+{
+    if ( buffer == NULL || len < 1 ) return 0;
+    if ( fd < 0 ) return 0;
+
+    size_t count;
+  
+    count = ::worker_read( fd, buffer, len );
+    
+    if ( count < 1 ) {
+        iseof = true;
+    }
+    return count;
+}
