@@ -523,16 +523,29 @@ FileEntry::typecandidate_t FileEntry::runTests( WCFiletype *type,
 
       if ( erg.type == NULL && type->getUseMagic() == true ) {
           erg.tests_done++;
-          if ( temp_results.magic_avail == false ) {
-              if ( worker_islocal( fullname ) == true ) {
-                  temp_results.magic_result = MagicDB::getInstance().getInfo( fullname );
-              } else {
-                  temp_results.magic_result = MagicDB::getInstance().getInfo( fbuffer, reads );
+          if ( type->getMagicCompressed() == true ) {
+              if ( temp_results.magic_avail_compressed == false ) {
+                  if ( worker_islocal( fullname ) == true ) {
+                      temp_results.magic_result_compressed = MagicDB::getInstance().getInfo( fullname, true );
+                  } else {
+                      temp_results.magic_result_compressed = MagicDB::getInstance().getInfo( fbuffer, reads, true );
+                  }
+                  temp_results.magic_avail_compressed = true;
+              }
+          } else {
+              if ( temp_results.magic_avail == false ) {
+                  if ( worker_islocal( fullname ) == true ) {
+                      temp_results.magic_result = MagicDB::getInstance().getInfo( fullname );
+                  } else {
+                      temp_results.magic_result = MagicDB::getInstance().getInfo( fbuffer, reads );
+                  }
+                  temp_results.magic_avail = true;
               }
-              temp_results.magic_avail = true;
           }
 
-          if ( type->magicMatch( temp_results.magic_result ) == true ) {
+          if ( type->magicMatch( ( type->getMagicCompressed() == true ) ?
+                                 temp_results.magic_result_compressed :
+                                 temp_results.magic_result ) == true ) {
               erg.prio = 3;
               erg.type = type;
           }
