@@ -1,6 +1,6 @@
 /* fontreq.cc
  * This file belongs to Worker, a filemanager for UNIX/X11.
- * Copyright (C) 2001-2006 Ralf Hoffmann.
+ * Copyright (C) 2001-2008 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -16,9 +16,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
-/* $Id: fontreq.cc,v 1.16 2005/08/17 06:29:36 ralf Exp $ */
 
 #include "fontreq.h"
 #include <aguix/aguix.h>
@@ -32,6 +31,63 @@
 #include <aguix/util.h>
 #include <aguix/request.h>
 
+#ifdef HAVE_XFT
+static const struct request_fonts_def {
+  const char *name;
+  FontRequester::request_fonts::type_t type;
+  const char *xname;
+  int size; } use_fonts[]={ { "default", FontRequester::request_fonts::FIXED, "Sans-10", 10},
+		            { "Sans", FontRequester::request_fonts::SCALABLE,
+			      "Sans-%d", 0},
+		            { "Serif", FontRequester::request_fonts::SCALABLE,
+			      "Serif-%d", 0},
+		            { "Mono", FontRequester::request_fonts::SCALABLE,
+			      "Mono-%d", 0}
+                         };
+#else
+static const struct request_fonts_def {
+  const char *name;
+  FontRequester::request_fonts::type_t type;
+  const char *xname;
+  int size; } use_fonts[]={ { "default", FontRequester::request_fonts::FIXED, "fixed", 8},
+		            { "9x15", FontRequester::request_fonts::FIXED, "9x15", 9},
+		            { "10x20", FontRequester::request_fonts::FIXED, "10x20", 10},
+		            { "12x24", FontRequester::request_fonts::FIXED, "12x24", 12},
+		            { "fixed", FontRequester::request_fonts::SCALABLE,
+			      "-misc-fixed-medium-r-*-*-%d-*-*-*-*-*-*-*", 0},
+		            { "fixed bold", FontRequester::request_fonts::SCALABLE,
+			      "-misc-fixed-bold-*-*-*-%d-*-*-*-*-*-*-*", 0},
+                            { "courier", FontRequester::request_fonts::SCALABLE,
+			      "-*-courier-medium-r-*-*-%d-*-*-*-*-*-*-*", 0},
+                            { "courier bold", FontRequester::request_fonts::SCALABLE,
+			      "-*-courier-bold-r-*-*-%d-*-*-*-*-*-*-*", 0},
+                            { "courier italic", FontRequester::request_fonts::SCALABLE,
+			      "-*-courier-medium-o-*-*-%d-*-*-*-*-*-*-*", 0},
+                            { "courier bold italic", FontRequester::request_fonts::SCALABLE,
+			      "-*-courier-bold-o-*-*-%d-*-*-*-*-*-*-*", 0},
+			    { "times", FontRequester::request_fonts::SCALABLE,
+			      "-*-times-medium-r-*-*-%d-*-*-*-*-*-*-*", 0},
+			    { "times bold", FontRequester::request_fonts::SCALABLE,
+			      "-*-times-bold-r-*-*-%d-*-*-*-*-*-*-*", 0},
+			    { "times italic", FontRequester::request_fonts::SCALABLE,
+			      "-*-times-medium-i-*-*-%d-*-*-*-*-*-*-*", 0},
+			    { "times bold italic", FontRequester::request_fonts::SCALABLE,
+			      "-*-times-bold-i-*-*-%d-*-*-*-*-*-*-*", 0},
+			    { "helvetica", FontRequester::request_fonts::SCALABLE,
+			      "-*-helvetica-medium-r-*-*-%d-*-*-*-*-*-*-*", 0},
+			    { "helvetica bold", FontRequester::request_fonts::SCALABLE,
+			      "-*-helvetica-bold-r-*-*-%d-*-*-*-*-*-*-*", 0},
+			    { "helvetica italic", FontRequester::request_fonts::SCALABLE,
+			      "-*-helvetica-medium-o-*-*-%d-*-*-*-*-*-*-*", 0},
+			    { "helvetica bold italic", FontRequester::request_fonts::SCALABLE,
+			      "-*-helvetica-bold-o-*-*-%d-*-*-*-*-*-*-*", 0},
+			    { "any font famliy", FontRequester::request_fonts::SCALABLE,
+			      "-*-*-medium-r-*-*-%d-*-*-*-*-*-*-*", 0},
+			    { "generic", FontRequester::request_fonts::SCALABLE,
+			      "-*-*-*-*-*-*-%d-*-*-*-*-*-*-*", 0}
+                          };
+#endif
+
 FontRequester::FontRequester(AGUIX *parent)
 {
   int nr_of_fonts=sizeof(use_fonts)/sizeof(struct request_fonts);
@@ -126,6 +182,7 @@ int FontRequester::request(const char *xname,char**return_xname)
   ac1_2->add( new Text( aguix, 0, 0, catalog.getLocale( 383 ), 1 ), 0, 0, cfix );
   ftext = (Text*)ac1_2->add( new Text( aguix, 0, 0, "", 1 ), 1, 0, cincwnr );
 
+#ifndef HAVE_XFT
   AContainer *ac1_4 = ac1->add( new AContainer( win, 4, 1 ), 0, 4 );
   ac1_4->setMinSpace( 5 );
   ac1_4->setMaxSpace( 5 );
@@ -135,6 +192,10 @@ int FontRequester::request(const char *xname,char**return_xname)
   missingtext = (Text*)ac1_4->add( new Text( aguix, 0, 0, "     ", 1 ), 1, 0, cincwnr );
   Button *missing_help_b = (Button*)ac1_4->add( new Button( aguix, 0, 0, catalog.getLocale( 483 ), 1, 0, 0 ),
                                                 3, 0, cfix );
+#else
+  ac1->setMinHeight( 0, 0, 4 );
+  ac1->setMaxHeight( 0, 0, 4 );
+#endif
 
   xb = (Button*)ac1->add( new Button( aguix, 0, 0, catalog.getLocale( 381 ), 1, 0, 0 ), 0, 5, cincw );
 
@@ -239,9 +300,11 @@ int FontRequester::request(const char *xname,char**return_xname)
 	    endmode=0;
           } else if(msg->button.button==cb) {
             endmode=1;
+#ifndef HAVE_XFT
           } else if ( msg->button.button == missing_help_b ) {
             req->request( catalog.getLocale( 124 ), catalog.getLocale( 714 ),
                           catalog.getLocale( 11 ) );
+#endif
           } else if(msg->button.button==xb) {
             buttonstr=(char*)_allocsafe(strlen(catalog.getLocale(11))+1+
                                         strlen(catalog.getLocale(8))+1);
@@ -397,7 +460,7 @@ void FontRequester::updatefont()
       font1=(struct request_fonts*)fontlist->getElementAt(i);
       if(font1!=NULL) {  
         tstr=font1->xname;
-        if(font1->type==request_fonts::SCALEABLE) {
+        if(font1->type==request_fonts::SCALABLE) {
           buffer = (char*)_allocsafe( strlen( tstr ) + A_BYTESFORNUMBER( int ) + 1 );
           sprintf(buffer,tstr,getsize(i,j));
           example_sg->setFont( buffer );
@@ -410,6 +473,7 @@ void FontRequester::updatefont()
           ftext->setText(tstr);
         }
 
+#ifndef HAVE_XFT
         missingtext->setText( "" );
         if ( xfontname.length() > 0 ) {
           AGUIXFont *font = aguix->getFont( xfontname.c_str() );
@@ -419,6 +483,7 @@ void FontRequester::updatefont()
             missingtext->setText( numbuf );
           }
         }
+#endif
         example_sg->changeHeightForCurFont();
   	ac1->setMinHeight( example_sg->getHeight(), 0, 2 );
 	ac1->setMaxHeight( example_sg->getHeight(), 0, 2 );
@@ -449,7 +514,7 @@ int FontRequester::findfont(const char *xn,int *return_i, int *return_j)
   i=0;
   while(font1!=NULL) {  
     tstr=font1->xname;
-    if(font1->type==request_fonts::SCALEABLE) {
+    if(font1->type==request_fonts::SCALABLE) {
       buffer = (char*)_allocsafe( strlen( tstr ) + A_BYTESFORNUMBER( int ) + 1 );
 
       for(j=0;j<FONTREQ_STEPS;j++) {
