@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifndef NORMALMODE_H
@@ -215,7 +215,7 @@ public:
                 NM_GETFILES_SELORACT} nm_getfiles_t;
   int getSelFiles( std::list<NM_specialsourceExt*>*, nm_getfiles_t, bool unselect = false );
   static void freeSelFiles( std::list<NM_specialsourceExt*>* );
-  int getDirSize( FileEntry *fe, loff_t &return_size );
+  int getDirSize( FileEntry *fe, loff_t &return_size, bool &cancel );
 
   virtual bool startdnd(DNDMsg *dm);
   virtual bool isyours( Widget *elem );
@@ -386,14 +386,23 @@ protected:
   bool shownextentry( const char *tstr, bool reverse_search = false, bool start_at_same = false );
 
   typedef struct changemod_info{
-    changemod_info();
-    bool forAll;
-    mode_t newmode;
+      changemod_info();
+      bool forAll;
+      
+      mode_t mode;
+      
+      typedef enum {
+          CHMOD_SET,
+          CHMOD_ADD,
+          CHMOD_REM
+      } chmod_op_t;
+      
+      chmod_op_t op;
   } changemod_info_t;
   int worker_changemod( const NM_specialsourceInt *ss1,
                         changemod_info_t *cminfo,
                         const NM_chmodorder *cmorder );
-  int applyNewMode( const NM_specialsourceInt *ss1, mode_t newmode );
+  int applyNewMode( const NM_specialsourceInt *ss1, changemod_info_t newmode );
   typedef struct changeown_info{
     changeown_info();
     bool forAll;
@@ -405,7 +414,7 @@ protected:
                         const NM_chownorder *coorder );
   int applyNewOwner( const NM_specialsourceInt *ss1, uid_t newuid, gid_t newgid );
 
-  int requestNewMode(FileEntry *fe,mode_t *return_mode);
+  int requestNewMode( FileEntry *fe, changemod_info_t *return_mode );
   void setFilters( const std::list<NM_Filter> &filters );
   void configureFilters( bool * tshowhidden, std::list<NM_Filter> &filters );
   void setLVC4Filter( FieldListView *filv, int row, NM_Filter &fi);
@@ -625,7 +634,6 @@ protected:
     std::map<request_choose_t, std::string> choose_strings;
   };
   Requests requests;
-  std::string formatStringToString( const char *format, ... );
 
   typedef enum { REG_COPY_OK, REG_COPY_SKIP, REG_COPY_CANCEL, REG_COPY_ERROR } reg_copy_erg_t;
   
@@ -676,6 +684,8 @@ protected:
   void lv_pressed( int row, bool open_under_mouse = true );
   void buildPopUpMenu( FileEntry *entry );
   void startLVPopUpAction( AGMessage *msg );
+
+  time_t m_last_statebar_update;
 };
 
 class NMFieldListViewDND:public FieldListViewDND
