@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "nwc_fsentry.hh"
@@ -173,4 +173,23 @@ namespace NWC
         return _dest_statbuf.size;
     }
 
+    dev_t FSEntry::stat_dev() const
+    {
+        return _statbuf.dev;
+    }
+
+    dev_t FSEntry::stat_dest_dev() const
+    {
+        return _dest_statbuf.dev;
+    }
+
+    ino_t FSEntry::stat_inode() const
+    {
+        return _statbuf.inode;
+    }
+
+    ino_t FSEntry::stat_dest_inode() const
+    {
+        return _dest_statbuf.inode;
+    }
 }
