@@ -1,6 +1,6 @@
 /* partspace.cc
  * This file belongs to Worker, a filemanager for UNIX/X11.
- * Copyright (C) 2001-2006 Ralf Hoffmann.
+ * Copyright (C) 2001-2008 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -16,12 +16,12 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "partspace.h"
 
-void *PS_slavestart( void *arg )
+static void *PS_slavestart( void *arg )
 {
   PartSpace *ps = (PartSpace*)arg;
   ps->slavehandler();
