@@ -347,9 +347,9 @@ SearchOp::run( WPUContext *wpu, ActionMessage *msg )
   }
   win->useStippleBackground();
 
-  _startdir_sg->selectAll();
-  _matchname_sg->selectAll();
-  _matchcontent_sg->selectAll();
+  _startdir_sg->selectAll( false );
+  _matchname_sg->selectAll( false );
+  _matchcontent_sg->selectAll( false );
 
   win->show();
   msg->getWorker()->setWaitCursor();
