@@ -1,6 +1,6 @@
 /* wconfig.h
  * This file belongs to Worker, a filemanager for UNIX/X11.
- * Copyright (C) 2001-2006 Ralf Hoffmann.
+ * Copyright (C) 2001-2008 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -16,9 +16,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
-/* $Id: wconfig.h,v 1.57 2006/02/09 07:24:23 ralf Exp $ */
 
 #ifndef WCONFIG_H
 #define WCONFIG_H
@@ -213,10 +212,7 @@ public:
   void clearLayoutOrders();
   void layoutAddEntry( layoutID_t nv );
 
-  std::vector<FlatTypeList::flattypelist_t> *getFlatTypeList();
-  void buildLVFromFlat( FieldListView *lv,
-			std::vector<FlatTypeList::flattypelist_t> *l1,
-			WCFiletype *highlightft );
+  FlatTypeList *getFlatTypeList();
   typedef enum { MOUSECONF_NORMAL_MODE,
 		 MOUSECONF_ALT_MODE } mouseconf_method_t;
   void setMouseSelectButton( int nv );
@@ -277,10 +273,17 @@ public:
 
       void setTextViewHighlightedBG( int col );
       int getTextViewHighlightedBG() const;
+
+      void setTextViewSelectionFG( int col );
+      int getTextViewSelectionFG() const;
+
+      void setTextViewSelectionBG( int col );
+      int getTextViewSelectionBG() const;
   private:
       struct {
           int textcolor, background;
           int highlightedfg, highlightedbg;
+          int selectionfg, selectionbg;
       } _textview_colors;
   };
 
@@ -288,6 +291,23 @@ public:
   ColorDef getColorDefs() const;
 
   int getAllCustomNames( std::vector<std::string> &custom_names );
+
+  void setSaveWorkerStateOnExit( bool nv );
+  bool getSaveWorkerStateOnExit() const;
+
+  typedef enum {
+      FONT_X11_GLOBAL,
+      FONT_X11_BUTTONS,
+      FONT_X11_LEFT,
+      FONT_X11_RIGHT,
+      FONT_XFT_GLOBAL,
+      FONT_XFT_BUTTONS,
+      FONT_XFT_LEFT,
+      FONT_XFT_RIGHT
+  } font_type_t;
+
+  void setFontName( font_type_t fonttype, const std::string &name );
+  std::string getFontName( font_type_t fonttype );
 protected:
   char *lang;
   unsigned int rows,columns;
@@ -432,6 +452,12 @@ protected:
   ColorDef _color_defs;
   
   std::string queryNewActionName();
+
+  bool m_save_worker_state_on_exit;
+
+  std::map< font_type_t, std::string > m_font_names;
+  void setFontNames( const std::map< font_type_t, std::string > &font_names );
+  std::map< font_type_t, std::string > getFontNames() const;
 };
 
 extern WConfig *wconfig;
