@@ -2718,17 +2718,29 @@ int Worker::runCommand( const char*exestr,
         if ( WIFEXITED( status ) ) {
           // normal exit
 #ifdef DEBUG
-          printf( "child normally exited!\n" );
+          printf( "child exited normally!\n" );
 #endif
           retval = WEXITSTATUS( status );
           break;
         } else if ( WIFSIGNALED( status ) ) {
 #ifdef DEBUG
           printf("child exited by a signal!\n");
+#endif
+          retval = -1;
+          break;
+        } else {
+#ifdef DEBUG
+          printf( "child exited by an unhandled way!\n" );
 #endif
           retval = -1;
           break;
         }
+      } else if ( ret == -1 ) {
+          if ( errno == ECHILD ) {
+              // child does not exists?
+              retval = -1;
+              break;
+          }
       }
       // wait some time and the retry
       waittime( 10 );
@@ -2767,6 +2779,9 @@ int Worker::runCommand( const char*exestr,
         if ( ret == p2 ) {
           doexit = true;
           continue;
+        } else if ( ret == -1 ) {
+            doexit = true;
+            continue;
         }
         if ( WorkerSHM::WSHMInit == true ) {
           if ( *(WorkerSHM::WSHMPtr) == 'Q' ) {
