@@ -1,6 +1,6 @@
 /* worker.h
  * This file belongs to Worker, a filemanager for UNIX/X11.
- * Copyright (C) 2001-2007 Ralf Hoffmann.
+ * Copyright (C) 2001-2008 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifndef WORKER_H
@@ -96,6 +96,10 @@ public:
 		  bool inbackground );
 
   static std::string getWorkerConfigDir();
+
+  void openWorkerMenu();
+
+  int getStatebarWidth();
  protected:
   void createMainWin();
   void setupMainWin();
@@ -120,7 +124,7 @@ public:
   Lister *lister[2];
   AWindow *mainwin;
   Button *aboutb,*configureb,*clockbar;
-  SolidButton *statebar;
+  Button *statebar;
   Button **pathbs;
   Button **buttons;
   AWindow *listerwin[2];
@@ -150,6 +154,7 @@ public:
   
   KeySym lastkey;
   unsigned int lastmod;
+  Time lastkey_time;
   
   typedef struct _keyhash {
     WCButton *b;
@@ -175,6 +180,11 @@ public:
   std::string last_statebar_text[2];
 
   std::string queryLanguage();
+
+  void createWorkerMenu();
+  class PopUpMenu *m_worker_menu;
+  void startMenuAction( AGMessage *msg );
+  void saveWorkerState();
 };
 
 #endif
