@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "worker.h"
@@ -186,6 +186,9 @@ FunctionProto *Worker::getCommand4ID(int id)
     case 53:
       fpr = new RunCustomAction();
       break;
+    case 54:
+      fpr = new OpenWorkerMenuOp();
+      break;
     default:
       fpr=new DNDAction();
   }
@@ -249,6 +252,7 @@ int Worker::getID4Command(FunctionProto *fpr)
   else if ( fpr->isName( DirBookmarkOp::name ) == true ) type = 51;
   else if ( fpr->isName( OpenContextMenuOp::name ) == true ) type = 52;
   else if ( fpr->isName( RunCustomAction::name ) == true ) type = 53;
+  else if ( fpr->isName( OpenWorkerMenuOp::name ) == true ) type = 54;
   return type;
 }
 
