@@ -24,7 +24,7 @@
 #include "worker.h"
 #include <aguix/utf8.hh>
 
-#define CATENTRY 811
+#define CATENTRY 813
 #define CATCOMENTRY 55
 #define CATFLAGENTRY 59
 
@@ -911,6 +911,8 @@ match any font." );
   catalog[808]=dupstring( "Use parent directory and activate entry" );
   catalog[809]=dupstring( "Path" );
   catalog[810]=dupstring( "Alias" );
+  catalog[811]=dupstring( "Decompress file" );
+  catalog[812]=dupstring( "(in order of access [newest is bottom])" );
 
 //  catalog[]=dupstring( "" );
   
