@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 /* $Id: wpucontext.cc,v 1.26 2005/05/19 19:54:55 ralf Exp $ */
 
@@ -890,7 +890,7 @@ struct WPUContext::ifp_erg *WPUContext::ifp_factor()
                 str1 += error;
                 _freesafe( reqstr );
 
-                AFontWidth lencalc( worker->getAGUIX(), NULL );
+                RefCount<AFontWidth> lencalc( new AFontWidth( worker->getAGUIX(), NULL ) );
                 TextStorageString ts( str1, lencalc );
 
                 worker->getRequester()->request( catalog.getLocale( 347 ), ts, catalog.getLocale( 11 ) );
@@ -952,7 +952,7 @@ struct WPUContext::ifp_erg *WPUContext::ifp_factor()
                 str1 += error;
                 _freesafe( reqstr );
 
-                AFontWidth lencalc( worker->getAGUIX(), NULL );
+                RefCount<AFontWidth> lencalc( new AFontWidth( worker->getAGUIX(), NULL ) );
                 TextStorageString ts( str1, lencalc );
 
                 worker->getRequester()->request( catalog.getLocale( 347 ), ts, catalog.getLocale( 11 ) );
